#ifndef _COLOR_H_
#define _COLOR_H_

#include <cmath>
#include <stdint.h>


/*
Color class: holds color information in floating point format
and defines mathematical operations on color components like
addition, subtraction etc. Constructors are provided for various
initialization and conversion cases as well as functions to extract
the color in packed 32bit mode 16bit mode etc.
*/

class Color {
public:
	float r, g, b, a;

	Color(float intensity = 1.0f);
	Color(float r, float g, float b, float a = 1.0f);
	Color(unsigned int r, unsigned int g, unsigned int b, unsigned int a = 255);
	Color(int r, int g, int b, int a = 255);
	Color(uint32_t pcol);
	Color(long pcol);

	Color operator +(const Color &col) const;
	Color operator -(const Color &col) const;
	Color operator *(const Color &col) const;
	Color operator *(float scalar) const;

	void operator +=(const Color &col);
	void operator -=(const Color &col);
	void operator *=(const Color &col);
	void operator *=(float scalar);

	uint32_t GetPacked32() const;
	uint16_t GetPacked16() const;
	uint16_t GetPacked15() const;
	uint8_t GetNearest8(const unsigned char **pal) const;
};

Color BlendColors(const Color &c1, const Color &c2, float t);

#endif	// _COLOR_H_
