#include <cstdlib>
#include <GL/glut.h>
#include "material.h"
#include "image.h"

// setup default values
Material::Material() {
	ambient = diffuse = Color(1.0f, 1.0f, 1.0f);
	specular = Color(1.0f, 1.0f, 1.0f);
	emissive = Color(0.0f, 0.0f, 0.0f);
	specular_power = 0.0f;
	additive = false;
	alpha = 1.0f;
	texmap = envmap = lightmap = 0;
}

void Material::SetGLMaterial() const {
	glMaterialfv(GL_FRONT, GL_AMBIENT, (float*)&ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, (float*)&diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, (float*)&specular);
	glMaterialfv(GL_FRONT, GL_EMISSION, (float*)&emissive);
	glMaterialf(GL_FRONT, GL_SHININESS, specular_power);
}

/////////// LoadTexture() function ////////////
// loads a texture from a bitmap file and returns the OpenGL texture ID
///////////////////////////////////////////////

Texture LoadTexture(char *fname) {

	unsigned long width, height;
	unsigned long *image = (unsigned long*)LoadImage(fname, &width, &height);

	Texture tex_id;
	glGenTextures(1, &tex_id);
	glBindTexture(GL_TEXTURE_2D, tex_id);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, width, abs(height), 0, GL_BGRA, GL_UNSIGNED_BYTE, image);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	return tex_id;
}


