#ifndef _TERRAIN_H_
#define _TERRAIN_H_

#include <list>
#include "3ddatastruct.h"

typedef int Entity;	// REMOVE (just to be able to compile before going into game entity code)

#define LEVEL_SIZE	80
#define CELL_SIZE	10

enum LevelCellState {CellEmpty, CellCorridor, CellStart, CellLava, CellMagic};

// info needed in each cell
struct LevelCell {
	std::list<Entity*> entities;
	LevelCellState state;
};


// a level class: holds information on the active game level, as well as the actual geometry
class Level {
private:
	LevelCell cells[LEVEL_SIZE][LEVEL_SIZE];

	Mesh maze, ceiling, lava;

	void GenerateGeometry();					// generates the geometry (maze) from cell data


public:

	Level();

	void GetStartCell(int *x, int *y);
	LevelCell *GetCellFromPos(const Vector3 &pos);
	Vector3 GetCellCenter(int x, int y);

	void LoadFromFile(const char *fname);		// loads a level from a text file (see level0.lvl)
	void Render();								// renders the level geometry
};

#endif	// _TERRAIN_H_
